@@ -128,4 +128,19 @@ class GameMode(IntEnum):
 	RANKED = 31
 	DUEL = 33
 	ARENA = 34
-	DOMINATION = 42
\ No newline at end of file
+	DOMINATION = 42
+
+
+class Game(IntEnum):
+	"""Enumerates the different Atelier801 games."""
+	INVALID = 0
+
+	TRANSFORMICE = 4
+	FORTORESSE = 6
+	BOUBOUM = 7
+	NEKODANCER = 15
+	DEADMAZE = 17
+
+	@classmethod
+	def _missing_(cls, value):
+		return cls.INVALID
