@@ -85,6 +85,12 @@ class Player:
 		self.ducking = False
 		self.jumping = False
 
+	def __repr__(self):
+		info = ' '.join([f'{field}={getattr(self, field)!r}' for field in ('id', 'pid') if getattr(self, field) != -1])
+		if info:
+			info = ' ' + info
+		return f'<Player username={self.username!r}{info}>'
+
 	@classmethod
 	def from_packet(cls, packet: Packet):
 		"""Reads a Player from a packet.
@@ -210,7 +216,7 @@ class Profile:
 
 		modeStats = []
 		for _ in range(packet.read8()):
-			modeStats.append((packet.read8(), packet.read32(), packet.read32(), packet.read8()))
+			modeStats.append((packet.read8(), packet.read32(), packet.read32(), packet.read16()))
 		self.stats = Stats(stats, modeStats)
 
 		self.equippedOrb = packet.read8()
@@ -218,6 +224,7 @@ class Profile:
 		for _ in range(packet.read8()):
 			self.orbs.add(packet.read8())
 
+		packet.readBool()
 		self.adventurePoints = packet.read32()
 
 
@@ -253,4 +260,4 @@ class Stats:
 		self.gatheredCheese = stats[3]
 		self.bootcamps = stats[5]
 
-		self.modeStats = modeStats # id, progress, progressLimit, imageId
\ No newline at end of file
+		self.modeStats = modeStats # id, progress, progressLimit, imageId
