@@ -1,3 +1,5 @@
+from typing import List, Optional, Set, Tuple, Union
+
 from aiotfm.packet import Packet
 
 
@@ -31,22 +33,22 @@ class Shop:
 		All shaman object available in the shop.
 	"""
 	def __init__(self, packet: Packet):
-		self.cheese = packet.read32()
-		self.fraise = packet.read32()
-		self.look = packet.readUTF()
+		self.cheese: int = packet.read32()
+		self.fraise: int = packet.read32()
+		self.look: str = packet.readUTF()
 
-		self.owned_items = set(Item.from_packet(packet) for _ in range(packet.read32()))
-		self.items = set(ShopItem.from_packet(packet) for _ in range(packet.read32()))
+		self.owned_items: Set[Item] = set(Item.from_packet(packet) for _ in range(packet.read32()))
+		self.items: Set[ShopItem] = set(ShopItem.from_packet(packet) for _ in range(packet.read32()))
 
-		self.full_outfits = set(Outfit.from_fashion(packet) for _ in range(packet.read8()))
-		self.outfits = set(Outfit.from_packet(packet, i) for i in range(packet.read16()))
+		self.full_outfits: Set[Outfit] = set(Outfit.from_fashion(packet) for _ in range(packet.read8()))
+		self.outfits: Set[Outfit] = set(Outfit.from_packet(packet, i) for i in range(packet.read16()))
 
-		self.owned_shaman_objects = set(
+		self.owned_shaman_objects: Set[OwnedShamanObject] = set(
 			OwnedShamanObject.from_packet(packet) for _ in range(packet.read16())
 		)
-		self.shaman_objects = set(ShamanObject.from_packet(packet) for _ in range(packet.read16()))
+		self.shaman_objects: Set[ShamanObject] = set(ShamanObject.from_packet(packet) for _ in range(packet.read16()))
 
-	def to_dict(self):
+	def to_dict(self) -> dict:
 		"""Export the shop into a serializable dict.
 		:return: :class:`dict`
 		"""
@@ -59,7 +61,7 @@ class Shop:
 			"items": items
 		}
 
-	def cost(self, outfit):
+	def cost(self, outfit: 'Outfit') -> Tuple[int, int, int]:
 		"""Compute and return the total price of an outfit.
 		:param outfit: :class:`aiotfm.shop.Outfit`
 		:return: :class:`tuple`[:class:`int`]
@@ -71,7 +73,11 @@ class Shop:
 		for item in outfit.items:
 			if item.id == 0:
 				continue
+
 			item = self.getItem(item)
+			if item is None:
+				continue
+
 			cheese += item.cheese if item.cheese < 1000001 else 0
 			if item.fraise == 0 and item.cheese < 1000001:
 				fraise_sup += item.cheese
@@ -87,7 +93,7 @@ class Shop:
 
 		return cheese, fraise, fraise_sup
 
-	def getItem(self, item):
+	def getItem(self, item: 'Item') -> Optional['ShopItem']:
 		"""Return the shop item with the same id.
 		:param item: :class:`aiotfm.shop.Item` the item you want to price of.
 		:return: :class:`aiotfm.shop.ShopItem` the item with the prices.
@@ -97,7 +103,7 @@ class Shop:
 				return i
 		return None
 
-	def category(self, id_):
+	def category(self, id_: int) -> Set['ShopItem']:
 		"""Return the items from a category.
 		:param id_: :class:`int` the category's id?
 		:return: :class:`set`[:class:`aiotfm.shop.Item`] the items.
@@ -128,14 +134,17 @@ class Item:
 	colors: :class:`int`
 		The item's colors.
 	"""
-	def __init__(self, category, id_, colors=None):
-		self.category, self.id = int(category), int(id_)
-		self.uid = self.category * 10000 + self.id
+	def __init__(self, category: int, id_: int, colors: Optional[List[int]] = None):
+		self.category: int = int(category)
+		self.id: int = int(id_)
+		self.uid: int = self.category * 10000 + self.id
 
-		self.colors = colors or []
+		self.colors: List[int] = colors or []
 
-	def __eq__(self, other):
-		return self.category == other.category and self.id == other.id
+	def __eq__(self, other: object):
+		if isinstance(other, Item):
+			return self.category == other.category and self.id == other.id
+		return NotImplemented
 
 	def __hash__(self):
 		return self.uid
@@ -166,13 +175,15 @@ class Item:
 		return cls(cat, id_, colors)
 
 	@classmethod
-	def parse(cls, cat, string):
+	def parse(cls, cat: int, string: Union[List[str], str]) -> 'Item':
 		"""Parse an Item from a string.
 		:param cat: :class:`int` the item's category.
 		:param string: :class:`str` the item.
 		:return: :class:`aiotfm.shop.Item`
 		"""
-		string = string.split('_')
+		if isinstance(string, str):
+			string = string.split('_')
+
 		id_ = int(string[0])
 		if len(string) == 1:
 			return cls(cat, id_)
@@ -192,7 +203,7 @@ class ShopItem(Item):
 		The item's colors.
 	is_new: :class:`bool`
 		True if it's a new item.
-	flag: :class:`int`
+	flags: :class:`int`
 		Contains the item's metadata.
 	cheese: :class:`int`
 		The item's price in cheese.
@@ -215,7 +226,7 @@ class ShopItem(Item):
 		The number of customizable colors the item has.
 	is_new: :class:`bool`
 		True if it's a new item.
-	flag: :class:`int`
+	flags: :class:`int`
 		Contains the item's metadata.
 	cheese: :class:`int`
 		The item's price in cheese.
@@ -224,19 +235,20 @@ class ShopItem(Item):
 	special: :class:`int`
 		The item's special data.
 	"""
-	def __init__(self, category, id_, colors, is_new, flag, cheese, fraise, special):
+	def __init__(
+		self, category: int, id_: int, colors: int, is_new: bool,
+		flags: int, cheese: int, fraise: int, special: int
+	):
 		super().__init__(category, id_)
 
-		self.nbr_colors = colors
-		self.is_new = is_new
-		self.flag = flag
-		self.cheese = cheese
-		self.fraise = fraise
-		self.special = special
-		if self.is_new:
-			print(category, id_, colors, is_new, flag, cheese, fraise, special)
+		self.nbr_colors: int = colors
+		self.is_new: bool = is_new
+		self.flags: int = flags
+		self.cheese: int = cheese
+		self.fraise: int = fraise
+		self.special: int = special
 
-	def to_dict(self):
+	def to_dict(self) -> dict:
 		"""Export the item into a serializable dict.
 		:return: :class:`dict`
 		"""
@@ -250,8 +262,8 @@ class ShopItem(Item):
 		if self.cheese == 1000001:
 			data['purchasable'] = False
 			data['cheese'] = 0
-		if self.flag == 13:
-			data['collector'] = self.flag == 13
+		if self.flags == 13:
+			data['collector'] = self.flags == 13
 		if self.is_new:
 			data['new'] = self.is_new
 		return data
@@ -277,7 +289,7 @@ class Outfit:
 		The outfit's look.
 	id_: Optional[:class:`int`]
 		The outfit's id.
-	flag: :class:`int`
+	flags: :class:`int`
 		Contains the outfit's metadata.
 
 	Attributes
@@ -286,19 +298,21 @@ class Outfit:
 		The outfit's look.
 	id: :class:`int`
 		The outfit's id.
-	flag: :class:`int`
+	flags: :class:`int`
 		Contains the outfit's metadata.
 
 	"""
-	def __init__(self, look, id_=-1, flag=-1):
-		self.look = look
-		self.id = id_
-		self.flag = flag
+	def __init__(self, look: str, id_: int = -1, flags: int = -1):
+		self.look: str = look
+		self.id: int = id_
+		self.flags: int = flags
 
-	def __eq__(self, other):
+	def __eq__(self, other: object):
 		if isinstance(other, str):
 			return self.look == other
-		return self.look == other.look
+		if isinstance(other, Outfit):
+			return self.look == other.look
+		return NotImplemented
 
 	def __hash__(self):
 		return hash(hash(self.look) + hash(self.id))
@@ -311,8 +325,8 @@ class Outfit:
 		"""
 		id_ = packet.read16()
 		look = packet.readUTF()
-		flag = packet.read8()
-		return cls(look, id_, flag)
+		flags = packet.read8()
+		return cls(look, id_, flags)
 
 	@classmethod
 	def from_packet(cls, packet: Packet, id_):
@@ -323,12 +337,12 @@ class Outfit:
 		return cls(packet.readUTF(), id_)
 
 	@property
-	def fur(self):
+	def fur(self) -> int:
 		"""The fur's id of the outfit."""
 		return int(self.look.split(';')[0])
 
 	@property
-	def items(self):
+	def items(self) -> List[Item]:
 		"""The outfit's items."""
 		items = []
 		for i, item in enumerate(self.look.split(';')[1].split(',')):
@@ -336,47 +350,47 @@ class Outfit:
 		return items
 
 	@property
-	def head(self):
+	def head(self) -> Item:
 		"""The outfit's head item."""
 		return self.items[0]
 
 	@property
-	def eyes(self):
+	def eyes(self) -> Item:
 		"""The outfit's eyes item."""
 		return self.items[1]
 
 	@property
-	def ears(self):
+	def ears(self) -> Item:
 		"""The outfit's ears item."""
 		return self.items[2]
 
 	@property
-	def mouth(self):
+	def mouth(self) -> Item:
 		"""The outfit's mouth item."""
 		return self.items[3]
 
 	@property
-	def neck(self):
+	def neck(self) -> Item:
 		"""The outfit's neck item."""
 		return self.items[4]
 
 	@property
-	def hair(self):
+	def hair(self) -> Item:
 		"""The outfit's hair item."""
 		return self.items[5]
 
 	@property
-	def tail(self):
+	def tail(self) -> Item:
 		"""The outfit's tail item."""
 		return self.items[6]
 
 	@property
-	def lenses(self):
+	def lenses(self) -> Item:
 		"""The outfit's lenses item."""
 		return self.items[7]
 
 	@property
-	def hands(self):
+	def hands(self) -> Item:
 		"""The outfit's hands item."""
 		return self.items[8]
 
@@ -392,7 +406,7 @@ class ShamanObject:
 		The number of customizable colors the object has.
 	is_new: :class:`bool`
 		The object's metadata.
-	flag: :class:`int`
+	flags: :class:`int`
 		Contains the object's metadata.
 	cheese: :class:`int`
 		The obect's pricein cheese.
@@ -407,7 +421,7 @@ class ShamanObject:
 		The number of customizable colors the object has.
 	is_new: :class:`bool`
 		The object's metadata.
-	flag: :class:`int`
+	flags: :class:`int`
 		Contains the object's metadata.
 	cheese: :class:`int`
 		The obect's pricein cheese.
@@ -415,16 +429,18 @@ class ShamanObject:
 		The obect's pricein fraise.
 
 	"""
-	def __init__(self, id_, colors, is_new, flag, cheese, fraise):
-		self.id = id_
-		self.colors = colors
-		self.is_new = is_new
-		self.flag = flag
-		self.cheese = cheese
-		self.fraise = fraise
+	def __init__(self, id_: int, colors: int, is_new: bool, flags: int, cheese: int, fraise: int):
+		self.id: int = id_
+		self.colors: int = colors
+		self.is_new: bool = is_new
+		self.flags: int = flags
+		self.cheese: int = cheese
+		self.fraise: int = fraise
 
-	def __eq__(self, other):
-		return self.id == other.id
+	def __eq__(self, other: object):
+		if isinstance(other, ShamanObject):
+			return self.id == other.id
+		return NotImplemented
 
 	def __hash__(self):
 		return self.id
@@ -463,13 +479,15 @@ class OwnedShamanObject:
 		The custom colors the object has.
 
 	"""
-	def __init__(self, id_, equiped, colors):
-		self.id = id_
-		self.equiped = equiped
-		self.colors = colors
+	def __init__(self, id_: int, equiped: bool, colors: List[int]):
+		self.id: int = id_
+		self.equiped: bool = equiped
+		self.colors: List[int] = colors
 
-	def __eq__(self, other):
-		return self.id == other.id
+	def __eq__(self, other: object):
+		if isinstance(other, OwnedShamanObject):
+			return self.id == other.id
+		return NotImplemented
 
 	def __hash__(self):
 		return self.id
@@ -484,4 +502,4 @@ class OwnedShamanObject:
 		equiped = packet.readBool()
 		colors = [packet.read32() for i in range(max(packet.read8() - 1, 0))]
 
-		return cls(id_, equiped, colors)
\ No newline at end of file
+		return cls(id_, equiped, colors)
